package sysinfo

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"os"
	"os/exec"
	"path/filepath"
	"strconv"
	"strings"
)

const (
	rootDir      = "/etc/sysinfo"
	snapshotJSON = "snapshot.json"
)

/* ------------------------------------------------------------------ */
/*  platform check (Ubuntu)                                           */
/* ------------------------------------------------------------------ */

func ensureUbuntu() error {
	out, err := runCmd("lsb_release", "-is")
	if err != nil {
		return fmt.Errorf("unsupported OS: lsb_release not found")
	}
	if strings.ToLower(strings.TrimSpace(string(out))) != "ubuntu" {
		return fmt.Errorf("unsupported OS: %s",
			strings.TrimSpace(string(out)))
	}
	return nil
}

/* ------------------------------------------------------------------ */
/*  shell helpers                                                     */
/* ------------------------------------------------------------------ */

func runCmd(name string, args ...string) ([]byte, error) {
	cmd := exec.Command(name, args...)
	var buf bytes.Buffer
	cmd.Stdout, cmd.Stderr = &buf, &buf
	err := cmd.Run()
	return buf.Bytes(), err
}

func safeRun(name string, args ...string) string {
	out, _ := runCmd(name, args...)
	return string(out)
}

func safeRead(path string) string {
	b, _ := ioutil.ReadFile(path)
	return string(b)
}

func emitJSON(f *os.File, v any) error {
	enc := json.NewEncoder(f)
	enc.SetIndent("", "  ")
	return enc.Encode(v)
}

/* ---------- generic helpers reused in collectors ------------------ */

func readNetStats(iface string) (json.RawMessage, error) {
	dir := filepath.Join("/sys/class/net", iface, "statistics")
	ent, err := ioutil.ReadDir(dir)
	if err != nil {
		return nil, err
	}
	m := map[string]int64{}
	for _, f := range ent {
		b, _ := ioutil.ReadFile(filepath.Join(dir, f.Name()))
		n, _ := strconv.ParseInt(strings.TrimSpace(string(b)), 10, 64)
		m[f.Name()] = n
	}
	return json.Marshal(m)
}
