package sysinfo

import (
	"os/exec"
)

// PackageManager abstracts distro package handling.
type PackageManager interface {
	Distro() string
	InstallPackages(pkgs ...string) error
	ExecExists(bin string) bool
}

/* ------------------------------------------------------------------ */
/*  aptManager (Ubuntu / Debian)                                      */
/* ------------------------------------------------------------------ */

type aptManager struct {
	logf    func(string, ...any) // logger from App
	updated bool                 // true after first apt-update
}

// newPackageManager is called once in app.go; the same instance is
// passed to every collector, so apt-update can be executed exactly once.
func newPackageManager(logf func(string, ...any)) PackageManager {
	return &aptManager{logf: logf}
}

func (a *aptManager) Distro() string { return "ubuntu" }

func (a *aptManager) ExecExists(bin string) bool {
	_, err := exec.LookPath(bin)
	return err == nil
}

func (a *aptManager) InstallPackages(pkgs ...string) error {
	if len(pkgs) == 0 {
		return nil
	}

	// refresh package index only the first time we install anything
	if !a.updated {
		a.logf("apt update")
		if err := exec.Command(
			"apt-get", "-qq", "-y", "update",
		).Run(); err != nil {
			return err
		}
		a.updated = true
	}

	args := append(
		[]string{"-qq", "-y", "--no-install-recommends", "install"},
		pkgs...,
	)
	cmd := exec.Command("apt-get", args...)
	cmd.Env = append(cmd.Env, "DEBIAN_FRONTEND=noninteractive")
	a.logf("apt install %v", pkgs)
	return cmd.Run()
}
